#ifndef _CFORMS_H
#define _CFORMS_H

#ifndef OK
#define OK	0
#endif

#ifndef FAIL
#define FAIL	-1
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#define EVENT_KEY	1
#define EVENT_REFRESH	2
#define EVENT_ENTRY	3
#define EVENT_EXIT	4
#define EVENT_DRAW	5
#define EVENT_PREVIOUS	6
#define EVENT_NEXT	7

#define FLD_TYPE	0x000F
#define FLD_MODIFIER	0x0FF0

#define FLD_RIGHT	0x0010
#define FLD_LEFT	0x0020
#define FLD_UPPERCASE	0x0040
#define FLD_PROTECTED	0x0080

#define FLD_INT		0x0001
#define FLD_STR		0x0002
#define FLD_ALNUM	0x0003

struct xy {
    int x, y;
};

#ifndef _COMP_H
struct event {
    int type;
    int code;
    void (*func)();
};

struct field {
    char *name;
    int type;
    int len;
    char *data;
    struct xy pos;
    char *lvalue;
    char *rvalue;
    struct event *event;
    int n_events;
    int flags;
    char modified;
    struct picture *picture;
};

struct literal {
    struct xy pos;
    char *value;
    int display_flags;
};

struct picture {
    char *name;
    struct field *field;
    int n_fields;
    struct literal *literal;
    int n_literals;
    struct event *event;
    int n_events;
    int x, y;
    int width, height;
};

struct module {
    struct picture *picture;
    int n_pictures;
    struct event *event;
    int n_events;
};

extern struct current {
    struct picture *picture;
    struct field *field;
} current;

/*
 * Start/stop functions.
 */
extern int cforms_init(void);
extern int cforms_end(void);

/*
 * Picture functions.
 */
extern struct picture *picture(char *fmt, ...);
extern int             pic_call(struct picture *pic, char *fieldname);
extern int             pic_clear(struct picture *pic);

/* 
 * Field functons.
 */
extern struct field *field(char *fmt, ...);
extern struct field *fld_move(struct field *fld);
extern struct field *fld_down(struct field *fld);
extern struct field *fld_up(struct field *fld);
extern struct field *fld_left(struct field *fld);
extern struct field *fld_right(struct field *fld);
extern struct field *fld_first(void);
extern struct field *fld_last(void);
extern struct field *fld_next(struct field *fld);
extern struct field *fld_previous(struct field *fld);
extern char         *fld_get(struct field *fld);
extern int           fld_set(struct field *fld, char *value);
extern int           fld_nset(struct field *fld, char *value, int n);
extern int           fld_len(struct field *fld);
extern int           fld_isempty(struct field *fld);
extern int           fld_ismodified(struct field *fld);
extern void          fld_touch(struct field *fld);

/*
 * General functions.
 */
extern                 get_fld(struct field *fld, int (*check)());
extern int             generate_event(int ev_type, int ev_code);
extern int             strequ(char *s1, char *s2);
extern int             message(char *fmt, ...);
extern int             message_nr(char *fmt, ...);


extern struct module _module;

#endif /* _COMP_H */

#endif /* _CFORMS_H */

